#//-----------------------------------------------
#//TUSB-INS8/8用サンプルプログラム
#//Python 用モジュール
#//
#//2024_05_01　株式会社タートル工業
#//Copyright (C) 2024 Turtle Industry Co.,Ltd.
#//-----------------------------------------------
import TUSBINS8
import sys

from SpFuncs import *

from functools import partial

DevId = None            # 開いているデバイスのID
mon_t = None            # タイマー管理クラスインスタンス
lock = threading.Lock() # デバイスコマンドが同時実行されないために使用するロック

# Sub Functions -----------------------------

# Widgetの表示を設定する
# stat -> 'Close' , 'Open'
def WidgetState(stat):
    """Widgetの表示を設定する
    
    Arguments:
        stat        ----    設定状態 'Close' or 'Open'
    """
    EnClose = tk.NORMAL if( stat == 'Close' ) else tk.DISABLED
    EnOpen = tk.NORMAL if( stat == 'Open' ) else tk.DISABLED
    IDCmbb['state'] = EnClose
    OpenBtn['state'] = EnClose
    CloseBtn['state'] = EnOpen
    global EdgeDirEn
    EdgeDirEn = ( stat == 'Open' )
    IntervalSpnb['state'] = EnOpen
    for bit in range(8):OutputChk[bit]['state'] = EnOpen
    if stat != 'Close' :
        form.protocol('WM_DELETE_WINDOW',lambda: 'pass' )
    else :
        form.protocol('WM_DELETE_WINDOW',lambda: form.destroy())

def EdgeSet():
    """エッジ機能を設定する"""
    Data = 0
    for bit in range(8): Data += (1 << bit) if EdgeDirV[bit] else 0 # ディレクション設定配列をビットパターン値に変換する
    Time = IntervalVal.get()
    if lock.acquire(timeout=1):
        ret = TUSBINS8.Tusbins88_EdgeSet(DevId,Data,Time)
        lock.release()
    if ret != 0:
        ShowInfo(TUSBINS8.GetErrMessage(ret))

# Timer -------------------------------------
def Monitor():
    """入力状態モニタータイマー"""
    global InputSt
    global EdgeSt
    # 入力状態を読み込む
    if lock.acquire(timeout=1):
        ret = TUSBINS8.Tusbins88_Read(DevId)
        lock.release()
    if ret[0] != 0:return
    for bit in range(8):
        if IsBitSet(ret[1],bit) != IsBitSet(InputSt,bit):
            SetLEDCanvas(InLedCanv[bit], IsBitSet(ret[1],bit))
    InputSt = ret[1]

    # エッジ検出状態を読み込む
    if lock.acquire(timeout=1):
        ret = TUSBINS8.Tusbins88_EdgeRead(DevId)
        lock.release()
    if ret[0] != 0:return
    for bit in range(8):
        if IsBitSet(ret[1],bit) != IsBitSet(EdgeSt,bit):
            SetLEDCanvas(EdgeLedCanv[bit], IsBitSet(ret[1],bit))
    EdgeSt = ret[1]

# Event Handler -----------------------------

def EdgeDir_on(event,Bit):
    """エッジ検出機能の方向が変更された時に呼び出される
    
    Arguments:
        event           ----    イベント
        Bit             ----    変更されたビット 0~15
    """
    if EdgeDirEn:
        EdgeDirV[Bit] = 0 if EdgeDirV[Bit] == 1 else 1
        SetUDCanvasArrow(EdgeDirCanv[Bit],EdgeDirV[Bit])
        EdgeSet()

def Interval_on():
    """エッジ検出インターバル設定が変更された
    """
    EdgeSet()

def Output_on():
    """出力設定(チェック)が変化"""
    Data = 0
    for bit in range(8):Data += (1<<bit) if OutputVal[bit].get() else 0
    if lock.acquire(timeout=1):
        ret = TUSBINS8.Tusbins88_Write(DevId,Data)
        lock.release()
    if ret != 0:
        ShowInfo(TUSBINS8.GetErrMessage(ret))

def OpenBtn_on():
    """オープンボタン押下"""
    global DevId
    DevId = IDCmbb.current()
    ret = TUSBINS8.Tusbins88_Device_Open(DevId)
    if ret ==0:
        WidgetState('Open')
        EdgeSet()
        global mon_t
        mon_t = intTT(0.5,Monitor)
        mon_t.Start()
    else:
        ShowInfo(TUSBINS8.GetOpenErrMessage(ret))

def CloseBtn_on():
    """クローズボタン押下"""
    mon_t.Stop()
    TUSBINS8.Tusbins88_Device_Close(DevId)
    WidgetState('Close')

# form generation ---------------------------
form  = tk.Tk()
form.geometry('370x482')
form.title('TUSB-INS8/8 Sample Program')
form.attributes("-toolwindow",True)

# Input Frame ###################
InputFrame = GetLFrame(form,"Input",0.9,0.48,0.05,0.0)
# 見出しラベル
GetLabel(InputFrame,"Bit",0.15,0.05)
GetLabel(InputFrame,"Input",0.28,0.05)
GetLabel(InputFrame,"Edge",0.48,0.05)
# ビット毎表示項目
InLedCanv =list()
InputSt = 0
EdgeDirCanv = list()
EdgeDirV = [1 for i in range(8)]
EdgeDirEn = False
EdgeLedCanv = list()
EdgeSt = 0
bypit = 0.1    # 行ピッチ
for bit in range(8):
    ly = 0.2+bypit*bit
    GetLabel(InputFrame,str(bit),0.15,ly)
    InLedCanv.append(GetLEDCanvas(InputFrame,0.31,ly))
    SetLEDCanvas(InLedCanv[bit],False)
    EdgeDirCanv.append(GetUDCanvas(InputFrame,0.53,ly,partial(EdgeDir_on,Bit=bit)))
    SetUDCanvasArrow(EdgeDirCanv[bit],EdgeDirV[bit])
    EdgeLedCanv.append(GetLEDCanvas(InputFrame,0.60,ly))
    SetLEDCanvas(EdgeLedCanv[bit],False)
#  凡例 Frame
LegendFrame = GetFrame(InputFrame,0.28,0.5,0.7,0.0)
SetUDCanvasArrow(GetUDCanvas(LegendFrame,0.15,0.15,None),0)
GetLabel(LegendFrame,"Off->On Edge",0.0,0.4)
SetUDCanvasArrow(GetUDCanvas(LegendFrame,0.15,0.65,None),1)
GetLabel(LegendFrame,"On->Off Edge",0.0,0.9)
#  Interval Frame
IntervalFrame = GetLFrame(InputFrame,"Interval",0.28,0.4,0.7,0.5)
GetLabel(IntervalFrame,"t=",0.1,0.1)
IntervalVal = tk.IntVar(value=0)
IntervalSpnb = GetSpinBoxN(IntervalFrame,0,255,1,IntervalVal,0.8,0.1,0.4,Interval_on)
GetLabel(IntervalFrame,"t+1[ms]",0.2,0.8)

# Output Frame ###################
OutputFrame = GetLFrame(form,"Output",0.4,0.48,0.05,0.5)
# 見出しラベル
GetLabel(OutputFrame,"Bit",0.2,0.05)
GetLabel(OutputFrame,"Output",0.4,0.05)
# ビット毎項目
OutputChk = list()
OutputVal = GenTkVarList(8,tk.BooleanVar,False)
bypit = 0.1   # 行ピッチ
for bit in range(8):
    ly = 0.2+bypit*bit
    GetLabel(OutputFrame,str(bit),0.2,ly)
    OutputChk.append(GetChkBoxCom(OutputFrame,"",0.4,ly,OutputVal[bit],partial(Output_on)))

#ID open close ###################
IDList = [str(i) for i in range(16)]
GetLabel(form,"ID",0.5,0.85)
IDCmbb = GetComboBox(form,IDList,0.15,0.55,0.85)
OpenBtn = GetButton(form,"DevOpen",OpenBtn_on,0.2,0.6,0.93)
CloseBtn = GetButton(form,"DevClose",CloseBtn_on,0.2,0.85,0.93)

# Start
WidgetState('Close')
form.mainloop()
